@@ -35,10 +35,11 @@ Basic example
     td = TDigest.compute(x, w, compression=1_000)
     # td now contains "compressed" distribution: centroids with their means and weights
 
-    # TDigest can be used to provide mean or approximate quantiles (i.e. inverse CDF):
+    # TDigest can be used to provide mean, variance or approximate quantiles (i.e. inverse CDF):
     td.mean
     quantiles = [0., 0.01, 0.05, 0.25, 0.5, 0.75, 0.95, 0.99, 1.]
     td.inverse_cdf(quantiles)
+    td.variance   # new in 0.1.4
 
     # these results are close to numpy values (note that numpy provides only unweighted quantiles)
     np.average(x, weights=w)  # mean should be exact
@@ -78,6 +79,9 @@ implementation. All other tested TDigest implementations in Python are much slow
     # PyTDigest, n = 1,000,000: 0.21 seconds
     # PyTDigest, n = 10,000,000: 2.02 seconds
 
+Since version 0.1.4, PyTDigest calculates also the exact variance of the data. Single addition is slightly slower
+now (~220 ns).
+
 Similar packages
 ----------------
 
@@ -141,6 +145,7 @@ batch updates into the C layer, so the iteration has to be done in python:
         tdcffi.insert(xx)
     end = time.time()
     print(f'TDigest-CFFI: {end - start:.3g} seconds')
+    # TDigest-CFFI: 0.479 seconds
 
 Hence, this package is still almost 20x slower than this package when used over numpy arrays. In addition, t-digest CFFI
 package allows only for integer weights.
@@ -199,4 +204,4 @@ from .pytdigest import TDigest, HandlingInvalid
 
 __all__ = ['TDigest', 'HandlingInvalid']
 __author__ = 'Tomas Protivinsky'
-__version__ = '0.1.2'
+__version__ = '0.1.3'
